/*
* DMX-Chaser-Controller mit AVR (Mega8)
* DMX-Transmitter-Modul v0.5 by Andreas Lang
* Copyright A. Lang 2007
*/
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#define F_OSC 16000000
#define USART_BAUD(F_OSC) ((F_OSC)/((250)*16)-1)

//Einstellungen BREAK-Dauer (min 88s, 125s empfohlen (und verwendet), kein oberer Grenzwert)
#define BRK_TCNT 6
#define BRK_PSCB 2

//Einstellungen MAB-Dauer (min 8s, max 1s, verwendet: 32s)
#define MAB_TCNT 192
#define MAB_PSCB 2

//Anzahl DMX-Kanle
#define DMX_CHAC 128

volatile uint8_t dmx_data[DMX_CHAC];
volatile uint8_t dmx_ptr=0;
volatile uint8_t dmx_tmrfunc=0;

SIGNAL(SIG_OVERFLOW0)
	{
	if (dmx_tmrfunc==0) //BREAK Ende, MAB start
		{
		TCCR0=(TCCR0&0xf8)|MAB_PSCB;
		TCNT0=MAB_TCNT;
		dmx_tmrfunc=1;
		PORTD|=(1<<1);//TxD High
	}else //MAB Ende
		{
		dmx_ptr=0;
		UCSRB|=(1<<TXEN)|(1<<TXCIE);//USART wieder einschalten
		TIMSK&=~(1<<TOIE0);
		UDR=0;
	}
}
SIGNAL(SIG_UART_TRANS)
	{
	if (dmx_ptr==DMX_CHAC)
		{
		TIFR|=(1<<TOV0);
		UCSRB&=~((1<<TXEN)|(1<<TXCIE));//USART ausschalten
		TCCR0=(TCCR0&0xf8)|BRK_PSCB;
		TCNT0=BRK_TCNT;
		dmx_tmrfunc=0; //BREAK wird erzeugt
		TIMSK|=(1<<TOIE0);
		UCSRB&=~(1<<TXCIE);
		PORTD&=~(1<<1); //TxD Low (wird bernommen, sobald USART fertig ist)
	}else
		UDR=dmx_data[dmx_ptr++];
}
SIGNAL(SIG_SPI)
	{ //SPI Data reception ISR
	static uint8_t data=0, stat=0, ch=0;
	data=SPDR;
	if (stat==0)
		{
		ch=data;
		if ((ch!=0)&&(ch<192))
			stat=1;
	}else
		{
		if (ch<=128)
			dmx_data[ch-1]=data;
		stat=0;
	}
}
void startdmx(void)
	{
	TIFR|=(1<<TOV0);//Clear old Interrupt flag
	TCCR0=(TCCR0&0xf8)|BRK_PSCB;
	TCNT0=BRK_TCNT;
	dmx_tmrfunc=0; //BREAK wird erzeugt
	TIMSK|=(1<<TOIE0); //enable tov0 interrupt
	PORTD&=~(1<<1); //TxD Low
}
void spi_init()
	{
	DDRB|=(1<<3);
	SPCR = (1<<SPE)|(1<<SPIE);
}
int main(void)
	{
	UBRRL=3;
	UBRRH=0;
	UDR    = 0;						 // clear USART data register
	UCSRA  = 0;						 // clear error flags, disable U2X and MPCM
	UCSRC = (1<<URSEL)|(1<<USBS)|(1<<UCSZ1)|(1<<UCSZ0);
	PORTD=0;
	sei();
	PORTD|=(1<<1)|(1<<2);
	DDRD|=(1<<1)|(1<<2);
	startdmx();
	spi_init();
	while(1)
		{
		for (uint8_t t=0; t<13;t++)
			_delay_ms(1);
	}
}
