/*
* DMX-Chaser-Controller mit AVR (Mega8)
* Audio-Analyse-Modul (HW_A)
*/
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <string.h>
#include "consoleui.h"
#include "keycodes.h"
#define F_OSC 16000000
// 250kBaud
#define USART_BAUD(F_OSC) ((F_OSC)/((250)*16)-1)


//Einstellungen BREAK-Dauer (min 88s, 125s empfohlen)
#define BRK_TCNT 5
#define BRK_PSCB 2

//EInstellungen MAB-Dauer (min 8s, max 1s)
#define MAB_TCNT 0
#define MAB_PSCB 1

//Anzahl DMX-Kanle
#define DMX_CHAC 50 

#define lpto 1000
#define lpto2 200

#include <avr/io.h>
#include <avr/pgmspace.h>
#include "ffft.h"		/* Defs for using Fixed-point FFT module */

#define	SYSCLK		16000000
#define BTO 2500

volatile uint16_t adcv;
volatile uint16_t pto=0;
volatile uint8_t plt=0,tlt=0,sendstat=0,esct=0;
volatile uint16_t taptime, lasttap, tto=0;
volatile uint16_t sbc=0, lsbd=0;
//volatile uint8_t dmx[32];
volatile uint8_t sample_rdy=0;
volatile uint8_t ledv=0,ledp=0;
volatile uint8_t frq[4];
volatile int16_t capture[FFT_N];			/* Wave captureing buffer */
complex_t bfly_buff[FFT_N];		/* FFT buffer */
uint16_t spektrum[FFT_N/2];		/* Spectrum output buffer */
volatile uint16_t bto=0,lsb=0,pot=0;
volatile uint8_t bright=0,blind=0,blubb=0;
volatile uint32_t sbcc=0;
volatile uint8_t mode=0,bcnt=0,sbto=0,bdto=0,sbe=0,mab=0;
volatile uint8_t txbuffer[64];
volatile uint8_t txwp=0,txrp=0,txs=0,txc=0;
volatile uint8_t rxbuffer[8], rxwp=0, rxrp=0;
volatile uint16_t kbbuffer[8];
volatile uint8_t kbwp=0, kbrp=0;
volatile uint8_t spic=0;
volatile uint8_t ch[64];
volatile uint8_t fade=0;
volatile uint8_t chaser[1][1];
volatile uint8_t chl=4, chp=0,beat=0;
volatile uint8_t xxx=0,col=0;
char hex(uint8_t val)
	{
	val&=0xf;
	if (val<=9)
		return '0'+val;
	else
		return 'A'+(val-10);
}

void sendchar(char foo)
	{
	while(txc>60);
	txbuffer[txwp++]=foo;
	txwp&=0x3f;
	txc++;
}
void sendstring(char* foo)
	{
	uint8_t p=0;
	while(foo[p]!=0)
		sendchar(foo[p++]);
}

void sendstring_P(const char *text)
	{
	char Zeichen;
	while (Zeichen = pgm_read_byte(text))
		{
		sendchar(Zeichen);
		text++;
	}
}

void sendu8_d(uint8_t foo)
	{
	uint8_t tmp=0;
	tmp=foo/100;
	if (tmp)
		sendchar('0'+tmp);
	tmp=(foo%100)/10;
	if (tmp)
		sendchar('0'+tmp);
	tmp=foo%10;
	sendchar('0'+tmp);
}

void setch( uint8_t ch, uint8_t val)
	{
	uint8_t foo;
	PORTB&=~(1<<4); // /SS low
	foo=SPSR;
	SPDR=ch;
	while (!(SPSR &(1<<SPIF)));
	foo=SPSR;
	SPDR=val;
	while (!(SPSR &(1<<SPIF)));
	PORTB|=(1<<4); // /SS high
}
void sendu8_h(uint8_t foo)
	{
	sendchar(hex(foo>>4));
	sendchar(hex(foo&0xf));
}

void sendhex(uint8_t foo)
	{
	sendchar(hex(foo>>4));
	sendchar(hex(foo&0xf));
}

void pushkey(uint16_t code)
	{
	kbbuffer[kbwp++]=code;
	kbwp&=0x7;
}
uint16_t popkey(void)
	{
	uint16_t tmp=0;
	if (kbwp!=kbrp)
		tmp=kbbuffer[kbrp++];
	kbrp&=0x7;
	return tmp;
}
SIGNAL(SIG_UART_TRANS)
	{
	if(txc)
		txc--;
	if (txwp!=txrp)
		{
		UDR=txbuffer[txrp++];
		txrp&=0x3f;
	}else
		txs=0;
}

SIGNAL(SIG_UART_RECV)
	{
	static uint8_t stat=0, hlp=0, foo=0;
	if (stat==0)
		{
		hlp=UDR;
		PORTC&=0x0f;
		if (hlp!=0) stat=1;
	}else
		{
		foo=UDR;
		if (hlp!=0)
			{
			if (hlp<50)
				{
				ch[hlp]=foo;
				PORTC&=0xf0;
			}
		}
		stat=0;
	}
	//rxbuffer[rxwp++]=UDR;
	//rxwp&=7;
}

SIGNAL(SIG_OVERFLOW2)
	{
	static uint8_t obtn=0, debounce=0;
	static uint16_t pwmcnt=0,btn=0,tmp=0;
	static uint8_t kstat=0;
	static uint8_t tic=0;
	tic++;
	tic&=15;
	if ((txs==0)&&(txrp!=txwp))
		{
		UCSRB &= ~(1<<TXCIE);
		UDR=txbuffer[txrp++];
		txs=1;
		UCSRB |= (1<<TXCIE);
	}

	//if ((tic==0)&&(fade<255))
	if (fade<255) fade++;
	//fade++;
	if (pto>0) pto--;
	if (tto>0) tto--;
	if (plt>0)
		{
		plt--;
		PORTD|=(1<<4);
	}
	else
		PORTD&=~(1<<4);
	if (tlt>0)
		{
		tlt--;
		PORTD|=(1<<5);
		ch[33]=255;
		ch[34]=255;
	}
	else
		{
		ch[33]=0;
		ch[34]=0;
		PORTD&=~(1<<5);
	}
	/*if (((PINB&0x04)==0)&&(sendstat==1))
		{
		sendisr();
	}*/
	if (mode==0)
		{
		PORTA|=(1<<3);
		PORTB&=~(1<<1);
	}else
		{
		PORTA&=~(1<<3);
		PORTB|=(1<<1);
	}
	if (bto>0)
		bto--;
	else
		bcnt=0;
	if (pto==0)
		{
		ledp>>=1;
		pto=lpto2;
	}
	if (sbe!=0)
		{
		PORTB&=~(1<<2);
		if (sbto!=0)
			PORTB|=(1<<3);
		else
			PORTB&=~(1<<3);
	}
	else
		{
		PORTB|=(1<<2);
		PORTB&=~(1<<3);
	}
	if (debounce==0)
		{
		btn=PINA&(1<<2);
		if ((btn==0)&&(obtn!=0))
			{
			sbe^=1;
			debounce=50;
		}
		obtn=btn;
	}
	if (debounce)
		debounce--;
	if (sbto>0)
		sbto--;
	if (bdto>0)
		bdto--;
	pwmcnt++;
	pwmcnt&=0x3ff;
	
	if (esct)
		esct--;
	else
		if (kstat!=0)
			{
			kstat=0;
			pushkey(KB_ESC);
		}
	if (rxwp!=rxrp)
		{
		tmp=rxbuffer[rxrp++];
		rxrp&=0x07;
		switch(kstat)
			{
			case 0:
				if(tmp==0x1b)
					{
					kstat=1;
					esct=100;
				}
				else
					pushkey(tmp);
				break;
			case 1:
				switch(tmp)
					{
					case 0x70:
						pushkey(KB_F1);
						kstat=0;
						break;
					case 0x71:
						pushkey(KB_F2);
						kstat=0;
						break;
					case 0x72:
						pushkey(KB_F3);
						kstat=0;
						break;
					case 0x73:
						pushkey(KB_F4);
						kstat=0;
						break;
					case 0x74:
						pushkey(KB_F5);
						kstat=0;
						break;
					case 0x75:
						pushkey(KB_F6);
						kstat=0;
						break;
					case 0x76:
						pushkey(KB_F7);
						kstat=0;
						break;
					case 0x77:
						pushkey(KB_F8);
						kstat=0;
						break;
					case 0x5b:
						kstat=2;
						break;
				}
				break;
			case 2:
				switch(tmp)
					{
					case 0x41:
						pushkey(KB_UP);
						break;
					case 0x42:
						pushkey(KB_DOWN);
						break;
					case 0x43:
						pushkey(KB_RIGHT);
						break;
					case 0x44:
						pushkey(KB_LEFT);
						break;
					case 0x53:
						pushkey(KB_LU);
						break;
					case 0x54:
						pushkey(KB_LD);
						break;
				}
				kstat=0;
		}
	}
	if (tic==0) 
		{
		xxx++;
		if (xxx==0)
			col+=18;
	}
}

SIGNAL(SIG_INTERRUPT0)
	{
	static uint16_t ctmr=0,osbc=0,tslsbc;
	ctmr=TCNT1;
	if (bdto==0)
		{
		plt=50;
		bto=BTO;
		if (bcnt<16)
			bcnt++;
			if (bcnt>2)
				{
				//if ((sbe!=0)&&(sbto==0)) beat=1;
				//sbto=50;
				osbc=sbc;
				sbc=(((uint32_t)sbc)+((uint32_t)(ctmr-lsbd)))/2;
				//if (OCR1B-ctmr)
				osbc=((OCR1B-ctmr)>>1)+(sbc>>1);
				//OCR1B=(ctmr+osbc)&0xffff;
				tslsbc=ctmr-lsb;
				//OCR1B=((lsb+osbc)>>1)+((sbc-tslsbc)>>1);
				
			}
		if (sbe==0) beat=1;
		/*
		sendstring("Beat timer: ");
		sendchar(48+(ctmr-lsbd)/10000);
		sendchar(48+((ctmr-lsbd)%10000)/1000);
		sendchar(48+((ctmr-lsbd)%1000)/100);
		sendchar(48+((ctmr-lsbd)%100)/10);
		sendchar(48+(ctmr-lsbd)%10);
		sendstring(" SBC: ");
		sendchar(48+sbc/10000);
		sendchar(48+(sbc%10000)/1000);
		sendchar(48+(sbc%1000)/100);
		sendchar(48+(sbc%100)/10);
		sendchar(48+sbc%10);
		sendstring("\n\r");*/
		lsbd=ctmr;
	}
	bdto=200;
}

SIGNAL(SIG_ADC)
	{
	static uint8_t stat=0;
	if (stat==FFT_N)
		{
		ADMUX=(1<<REFS0)|0x06;
		sample_rdy=1;
		stat++;
	}else	if (stat==(FFT_N+1))
		{
		adcv=ADC;
		ADMUX=(1<<REFS0)|0x01;
		stat++;
	}else if (stat==(FFT_N+2))
		{
		stat=0;
		ADMUX=(1<<REFS0)|0x00;
		pot=ADC>>1;
		if (pot<256)
			bright=pot;
		else
			bright=255;
		
		if (pot>=256)
			blind=pot-256;
		else
			blind=0;
		if (bright>=240)
			bright=255;
		if (blind<16)
			blind=0;
		if (sample_rdy==1)
			stat=FFT_N-1;
		else
			stat=0;
	}else
		{
		capture[stat]=ADC-32768;
		stat++;
	}
	ADCSRA|=(1<<ADSC);
}

SIGNAL(SIG_INTERRUPT1)
	{
	taptime=TCNT1-lasttap;
	OCR1A=TCNT1+taptime;
	tlt=50;
	lasttap=TCNT1;
}

SIGNAL(SIG_OUTPUT_COMPARE1A)
	{
	if ((PIND&(1<<6))!=0)
		tlt=50;
	OCR1A=TCNT1+taptime;
}
SIGNAL(SIG_OUTPUT_COMPARE1B)
	{
	lsb=TCNT1;
	sbto=50;
	OCR1B=lsb+sbc;
	if (sbe) beat=1;
}

SIGNAL(SIG_SPI)
	{
	spic=1;
	mode^=1;
}

void main(void)
	{
	ADCSRA=(1<<ADEN)|6;
	DDRC=255;
	ADMUX=(1<<REFS0)|0x06;
	ADCSRA|=(1<<ADSC)|(1<<ADIE);
	TCCR2=3;
	TIMSK|=(1<<TOIE2);
	MCUCR|=(1<<ISC01);
	MCUCR|=(1<<ISC11);
	GICR|=(1<<INT0)|(1<<INT1);
	sei();
	uint16_t cadcv=0;
	DDRD|=(1<<4)|(1<<5);
	PORTD|=(1<<3);
	TCCR1B=5;
	TIMSK|=(1<<OCIE1A)|(1<<OCIE1B);
	taptime=32000;
	PORTB|=(1<<0);
	DDRA=0;
	UBRRL  = 25;//51 USART_BAUD(F_OSC); (19200)
	UBRRH  = 0;
	UCSRC  = (1<<URSEL)|(3<<UCSZ0); // 8bit, 1 stop bit, asynchron 
	UDR    = 0;						 // clear USART data register
	UCSRA  = 0;						 // clear error flags, disable U2X and MPCM
	UCSRB  = (1<<RXEN)|(1<<TXEN)|(1<<TXCIE)|(1<<RXCIE);
	DDRD|=2;
	PORTD|=(1<<7)|(1<<6);
	DDRA|=(1<<3);
	PORTA|=(1<<2);
	DDRB|=(1<<1);
	DDRB|=(1<<3)|(1<<2);
	DDRA|=(1<<4);
	sendstat=1;
	PORTA|=(1<<4);
	sbc=32000;
	for(uint8_t i=0;i<255;i++)
		_delay_ms(1);
	PORTB|=(1<<4);
	DDRB|=(1<<7)|(1<<5)|(1<<4);
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<SPR0);
	//|(1<<SPIE);
	for(uint8_t i=0;i<128;i++)
		setch(i,0);
	//initmenu();
			//setch(1,64);
			//setch(3,64);
	menu(0);
	uint8_t chp=0;
	uint16_t ctmp=0;
	ch[41]=255;
	ch[42]=255;
	ch[3]=162;
	while(1)
		{
		ch[5]=0;
		for (uint8_t k=0;k<4;k++)
			{
			//ch[8*k+5]=200;
			ch[8*k+1]=xxx;
			ch[8*k+2]=255-xxx;
			if (blind==0)
				ch[8*k+3]=bright;
			else
				ch[8*k+3]=col;
			ch[8*k+4]=blind;
			
			if (sbe!=0)
				if ((PIND&(1<<7))==0)
					ch[8*k+5]=220;
				else
					ch[8*k+5]=255;
			else
				ch[8*k+5]=0;	
		}
		
		ctmp++;
		if (ctmp==50) ctmp=1;
		setch(ctmp,ch[ctmp]);
		if (beat==1)
			{
			UDR=66;
		}
	}
}
